<?php

class PayshopProcessCard
{
    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * @var PayshopClient
     */
    private $payshopSDK;

    /**
     * Class constructor
     */
    public function __construct() {
        add_action('woocommerce_api_payshop_process_card', [$this, 'execute']);

        $this->payshopUpdateOrder = new PayshopUpdateOrder();
        $this->payshopSDK = PayshopClientFactory::getInstance();
    }

    /**
     * Payment process with credit card
     *
     * @return void
     */
    public function execute()
    {
        try {
            $instrument = $this->getInstrument();
            
            $orderStatus = $this->getOrderStatus($instrument);
            $orderId = $_GET['order_id'];
            
            $this->payshopUpdateOrder->execute(
                $orderId,
                $orderStatus,
                $instrument['charge']['id'],
                $instrument['id'],
                $this->getPaymentId($instrument)
            );

            $this->redirect($instrument);
        } catch (\Throwable $e) {
            PayshopLog::generate($e->getMessage(), 'error');

            $this->failedRedirect(__('Error processing payment', 'payshop'));
        }

        exit;
    }

    /**
     * Get instrument information from the request
     *
     * @return array
     * @throws Exception
     */
    private function getInstrument()
    {
        $instrumentId = isset($_GET['instrumentId']) ? $_GET['instrumentId'] : null;

        $message = __('Error while getting instrument');

        if ($instrumentId) {
            $response = $this->payshopSDK->getInstrument($instrumentId);

            if ($response['status'] != 200) {
                throw new \Exception($message, 'payshop');
            }

            return $response['response'];
        }

        throw new \Exception($message, 'payshop');
    }

    /**
     * Get order status from the instrument response
     *
     * @param array $instrument
     * @return string
     */
    private function getOrderStatus($instrument)
    {
        $paymentInTwoSteps = $instrument['capture_on_creation'] == false;
        $instrumentSuccessCreated = $instrument['success'] == true;
        $authorizedPayment = $instrument['status'] == 'authorized';

        if ($paymentInTwoSteps &&$instrumentSuccessCreated && $authorizedPayment) {
            return 'processing';
        }

        $issetPayment = isset($instrument['last_payment']);
        $paymentSucess = $issetPayment && $instrument['last_payment']['success'];
        $paymentStatusSucess = $issetPayment && $instrument[ 'last_payment']['status'] == 'success';

        if ($paymentSucess && $paymentStatusSucess) {
            return 'processing';
        }

        return 'failed';
    }

    /**
     * Get payment id from the instrument response
     *
     * @param array $instrument
     * @return string|null
     */
    private function getPaymentId($instrument)
    {
        if (isset($instrument['last_payment'])) {
            return $instrument['last_payment']['id'];
        }

        return null;
    }

    /**
     * Redirect user based on the order status
     *
     * @param array $instrument
     * @return void
     */
    private function redirect($instrument)
    {
        if ($this->getOrderStatus($instrument) == 'processing') {
            $url = $_GET['success_url'];

            header('Location: ' . $url);
            return;
        } 
        
        $this->failedRedirect(__('Transaction error, check your payment informations', 'payshop'));
    }

    /**
     * Redirect user to the failed page
     *
     * @param string $message
     * @return void
     */
    private function failedRedirect($message)
    {
        wc_add_notice($message, 'error');

        header('Location: ' . wc_get_checkout_url() . '?message=' . $message);
        return;
    }
}
