<?php

class PayshopCreateCharge
{
    /**
     * @var PayshopClient
     */
    private $payshopSDK;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->payshopSDK = PayshopClientFactory::getInstance();
    }

    /**
     * Send charge and instrument to payshop
     *
     * @param string $paymentMethod
     * @param int $orderId
     * @param string $successURL
     * @return int
     * @throws Exception
     */
    public function execute($paymentMethod, $orderId, $successURL)
    {
        $this->validateCredentials();

        $order = wc_get_order($orderId);
        $this->validateCurrency($order);

        $total = (float) $order->get_total();    
        $shopName = get_bloginfo('name');

        $response = $this->payshopSDK->createCharge([
            'charge_type' => $paymentMethod,
            'amount' => $total,
            'currency' => 'EUR',
            'description' => $shopName . __(' order #', 'payshop') . $orderId,
            'events_url' => $this->webHookProcessEventURL(),
            'redirect_url' => $this->webhookProcessCardURL($orderId, $successURL),
            'instrument_params' => $this->getInstrumentParams($paymentMethod, $order) + ['description' => $shopName]
        ]);
    
        PayshopCommons::checkResponse($response);

        return $response['response']['id'];
    }

    /**
     * Validate credentials
     *
     * @return void
     * @throws Exception
     */
    private function validateCredentials()
    {
        if (!$this->payshopSDK) {
            $message = __('Check your credentials on Payshop setting page', 'payshop');
            PayshopLog::generate($message, PayshopLog::LOG_SEVERITY_ERROR);
            throw new Exception($message);
        }
    }

    /**
     * Validate currency
     *
     * @param WC_Order $order
     * @return void
     * @throws Exception
     */
    private function validateCurrency($order)
    {
        $currecy = $order->get_currency();

        if ($currecy != 'EUR') {
            $message = __('Product currency must be EUR', 'payshop');
            PayshopLog::generate($message, PayshopLog::LOG_SEVERITY_ERROR);
            throw new Exception($message);
        }
    }

    /**
     * Get instrument params for the payment method
     *
     * @param string $paymentMethod
     * @param WC_Order $order
     * @return array
     */
    private function getInstrumentParams($paymentMethod, $order)
    {
        if ($paymentMethod != 'multibanco' && $paymentMethod != 'payshop_reference') {
            $params = ['enable3ds' => true];

            $paymentInTwoSteps = $order->get_meta('_payshop_payment_in_two_steps', true);

            if ($paymentInTwoSteps == 'yes') {
                $params['recurring'] = false;
                $params['funds_control'] = 'authorize';
            }

            return $params;
        }

        if ($paymentMethod == 'payshop_reference') {
            $paymentSettings = get_option('woocommerce_payshop_reference_settings');
            $qtdDaysToExpire = isset($paymentSettings['payshop_reference_expiration_days']) ? $paymentSettings['payshop_reference_expiration_days'] : 2;
        }

        if ($paymentMethod == 'multibanco') {
            $paymentSettings = get_option('woocommerce_payshop_multibanco_settings');
            $qtdDaysToExpire = isset($paymentSettings['payshop_multibanco_expiration_days']) ? $paymentSettings['payshop_multibanco_expiration_days'] : 2;
        }

        $qtdDaysToExpire = $qtdDaysToExpire > 0 ? $qtdDaysToExpire : 2;

        return ['end_date' => date('Y-m-d', strtotime('+' . $qtdDaysToExpire . ' days'))];
    }

    /**
     * Get fake site url for development
     *
     * @return string
     */
    private function getSiteUrl()
    {
        return str_replace('http://127.0.0.1', 'https://eltonfonseca.dev', get_site_url());
    }

    /**
     * Get webhook process events url
     *
     * @return string
     */
    private function webHookProcessEventURL()
    {
        return $this->getSiteUrl() . '?wc-api=payshop_process_webhook';
    }

    /**
     * Get webhook process card url
     *
     * @param int $orderId
     * @param string $successURL
     * @return string
     */
    private function webhookProcessCardURL($orderId, $successURL)
    {
        $successURL = str_replace('http://127.0.0.1', 'https://eltonfonseca.dev', $successURL);

        return vsprintf('%s?wc-api=payshop_process_card&order_id=%s&success_url=%s', [
            $this->getSiteUrl(),
            $orderId,
            $successURL
        ]);
    }
}