<?php

class PayshopCreatePayment
{
    /**
     * @var PayshopClient
     */
    private $payshopSDK;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->payshopSDK = PayshopClientFactory::getInstance();
    }

    /**
     * Confirm payment in two steps on Payshop
     * 
     * @param WC_Order $order
     * @param float $effectiveValue
     * @return bool|void
     */
    public function execute($order, $effectiveValue)
    {
        try {
            $response = $this->payshopSDK->createPayment([
                'instrument' => $order->get_meta('_payshop_instrument_id'),
                'amount' => $effectiveValue,
                'currency' => 'EUR',
            ]);

            $this->validateResponseStatus($order, $response);
            $this->validatePaymentSuccess($order, $response);

            $this->updateOrder($order, $response, $effectiveValue);
        } catch (\Exception $e) {
            return false;
        }
    }

    /**
     * Validate response status
     * 
     * @param WC_Order $order
     * @param array $response
     * @return bool
     */
    private function validateResponseStatus($order, $response)
    {
        $message = __('Error on send confirmation payment to Payshop. Please, try again', 'payshop');

        if ($response['status'] != 201) {
            $notice[$order->get_id()] = $message;
            update_option('payshop_validation_message', $notice);
            
            throw new Exception($message);
        }

        return true;
    }

    /**
     * Validate payment success
     * 
     * @param WC_Order $order
     * @param array $response
     * @return bool|void
     * @throws Exception
     */
    private function validatePaymentSuccess($order, $response)
    {
        $message = __('Payment confirmation not accepted from Payshop', 'payshop');

        $successful = $response['response']['success'];
        $statusSuccess = $response['response']['status'] == 'success';

        if (!$successful || !$statusSuccess) {
            $notice[$order->get_id()] = $message;
            update_option('payshop_validation_message', $notice);

            throw new Exception($message);
        }

        return true;
    }

    /**
     * Update order
     * 
     * @param WC_Order $order
     * @param array $response
     * @param float $effectiveValue
     * @return void
     */
    private function updateOrder($order, $response, $effectiveValue)
    {
        $paymentId = $response['response']['id'];

        $order->update_meta_data('_payshop_payment_id', $paymentId, true);
        $order->update_meta_data('_payshop_effective_value', $effectiveValue, true);
        
        $order->save();
    }
}