<?php

class PayshopConfirmPayment
{
    /**
     * @var PayshopCreatePayment
     */
    private $payshopCreatePayment;

    /**
     * Class constructor
     */
    public function __construct()
    {
        $this->payshopCreatePayment = new PayshopCreatePayment();
    }

    /**
     * Create payment confirmation metabox
     * 
     * @return void
     */
    public function execute()
    {
        add_action('add_meta_boxes', [$this, 'addMetaBox']);
        add_action('save_post', [$this, 'saveMetaBox']);
        add_action('admin_notices', [$this, 'showValidationError'], 0);
    }
    
    /**
     * Add metabox
     * 
     * @return void
     */
    public function addMetaBox()
    {
        if (!$this->getOrder()) {
            return;
        }

        if ($this->isNotPayshopCard() || $this->isNotPaymentInTwoSteps()) {
            return;
        }

        if ($this->orderIsNotPaid() || $this->paymentIsCompleted()) {
            return;
        }
        
        if ($this->passedDateLimitToConfirmPayment()) {
            return;
        }
    
        add_meta_box(
            'payshop-confirm-payment',
            __('Confirm Payment', 'payshop'),
            [$this, 'renderMetaBox'],
            'shop_order',
            'side',
            'core'
        );
    }

    /**
     * Render metabox
     * 
     * @return void
     */
    public function renderMetaBox()
    {
        // Used on template
        $order = $this->getOrder();

        require_once PAYSHOP_PLUGIN_DIR . '/templates/views/admin/confirm-payment.php';
    }

    /**
     * Save metabox
     * 
     * @return void
     */
    public function saveMetaBox($postId)
    {
        if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
            return $postId;
        }

        if (!isset($_POST['nonce']) || !wp_verify_nonce($_POST['nonce'], 'payshop_confirm_payment')) {
            return $postId;
        }

        if (!isset($_POST['confirm-payment']) || $_POST['confirm-payment'] != 1) {
            return;
        }

        $effectiveValue = isset($_POST['effective-value']) ? $_POST['effective-value'] : null;

        if($effectiveValue == null) {
            $notice[$postId] = __('Effective value is required', 'payshop');
            update_option('payshop_validation_message', $notice);
            return;
        }

        if (!is_numeric($effectiveValue)) {
            $notice[$postId] = __('Effective value is invalid', 'payshop');
            update_option('payshop_validation_message', $notice);
            return;
        }

        $order = $this->getOrder();
        if ($effectiveValue > $order->get_total()) {
            $notice[$postId] = __('Effective value must be less than order value', 'payshop');
            update_option('payshop_validation_message', $notice);
            return;
        }

        $this->payshopCreatePayment->execute($order, $effectiveValue);
}
    
    /**
     * Get order
     * 
     * @return WC_Order
     */
    private function getOrder()
    {
        global $post;

        if (!$post) {
            return;
        }

        $order = wc_get_order($post->ID);
        if (!$order) {
            return;
        }

        return $order;
    }

    /**
     * Check date limit to confirm payment
     * 
     * @return bool
     */
    private function passedDateLimitToConfirmPayment()
    {
        $order = $this->getOrder();

        $dateLimitToConfirmPayment = PayshopCommons::getDateLimitToConfirmPayment($order);
        $currentDate = new DateTime();

        return $currentDate > $dateLimitToConfirmPayment;
    }

    /**
     * Check if is not payshop card
     * 
     * @return bool
     */
    private function isNotPayshopCard()
    {
        $order = $this->getOrder();

        $paymentMethod = $order->get_payment_method();

        return $paymentMethod !== 'payshop_card';
    }

    /**
     * Check if is not payment in two steps
     * 
     * @return bool
     */
    private function isNotPaymentInTwoSteps()
    {
        $order = $this->getOrder();

        $paymentInTwoSteps = $order->get_meta('_payshop_payment_in_two_steps');
        
        return $paymentInTwoSteps !== 'yes';
    }

    /**
     * Check if order is not paid
     * 
     * @return bool
     */
    private function orderIsNotPaid()
    {
        $order = $this->getOrder();

        return $order->get_status() !== 'processing' && $order->get_status() !== 'completed';
    }

    /**
     * Check if paymente is confirmed
     * 
     * @return bool
     */
    private function paymentIsCompleted()
    {
        $order = $this->getOrder();

        $payshopEffectiveValue = $order->get_meta('_payshop_effective_value');

        return !!$payshopEffectiveValue;
    }

    /**
     * Show validation error
     * 
     * @return void
     */
    public function showValidationError()
    {
        global $post;

        $notice = get_option('payshop_validation_message');

        if (empty($notice)) {
            return;
        }

        foreach ($notice as $pid => $message) {
            if ($post->ID == $pid) {
                echo '<div id="message" class="error"><p>' . $message . '</p></div>';
                unset($notice[$pid]);
                update_option('payshop_validation_message', $notice);
                break;
            }
        }
    }
}