<?php

class PayshopMultibanco extends WC_Payment_Gateway 
{
    /**
     * @var PayshopCreateCharge
     */
    private $payshopCreateCharge;

    /**
     * @var PayshopCreateInstrument
     */
    private $payshopCreateInstrument;

    /**
     * @var PayshopUpdateOrder
     */
    private $payshopUpdateOrder;

    /**
     * Define the payment gateway properties
     */
    public function __construct()
    {
        $this->id = 'payshop_multibanco';
        $this->icon = PAYSHOP_PLUGIN_URL . 'templates/img/multibanco.png'; 
        $this->has_fields = true; 
        $this->method_title = __('Multibanco', 'payshop');
        $this->method_description = __('Pay with Multibanco Reference', 'payshop'); 

        $this->supports = [
            'products'
        ];

        $this->init_form_fields();
        $this->init_settings();
        $this->title = $this->get_option('payshop_multibanco_title');
        $this->description = $this->get_option('payshop_multibanco_description');
        $this->enabled = $this->get_option('payshop_multibanco_enabled');
     
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);

        $this->payshopCreateCharge = new PayshopCreateCharge();
        $this->payshopCreateInstrument = new PayshopCreateInstrument();
        $this->payshopUpdateOrder = new PayshopUpdateOrder();
    }

    /**
     * Define configuration fields for the payment gateway
     * 
     * @return void
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'payshop_multibanco_enabled' => array(
                'title'       => __('Enable/Disable', 'payshop'),
                'label'       => __('Enable Multibanco Reference Payment', 'payshop'),
                'type'        => 'checkbox',
                'description' => '',
                'default'     => 'no'
            ),
            'payshop_multibanco_expiration_days' => array(
                'title'       => __('Days to reference expire', 'payshop'),
                'type'        => 'text',
                'description' => __('Number of days for Multibanco reference to expire.', 'payshop'),
                'default'     => '2',
            ),
            'payshop_multibanco_title' => array(
                'title'       => __('Title'),
                'type'        => 'text',
                'description' => __('This controls the title which the user sees during checkout.', 'payshop'),
                'default'     => __('Multibanco Reference', 'payshop'),
            ),
            'payshop_multibanco_description' => array(
                'title'       => __('Description'),
                'type'        => 'textarea',
                'description' => __('This controls the description which the user sees during checkout.', 'payshop'),
                'default'     => __('Pay with Multibanco reference', 'payshop'),
            ),
        );
    }
    
    /**
     * Display the payment fields on the checkout page
     * 
     * @return void
     */
    public function payment_fields()
    {
        if ($this->description) {
            if (get_option('payshop_environment') == 'sandbox') {
                $this->description .= '<br>' . __('TEST MODE ENABLED.', 'payshop');
            }

            echo wpautop(wp_kses_post($this->description));
        }

        do_action('woocommerce_payshop_multibanco_form_start', $this->id);
        echo __('After click on confirmation button. You will see reference on the next page', 'payshop');
        do_action('woocommerce_payshop_multibanco_form_end', $this->id);
    }

    /**
     * Process the payment
     * 
     * @param int $orderId
     * @return array|void
     */
    public function process_payment($orderId)
    {
        try {
            $order = wc_get_order($orderId);

            $chargeId = $this->payshopCreateCharge->execute(
                'multibanco', 
                $orderId, 
                $this->get_return_url($order)
            );

            $instrument = $this->createInstrument($chargeId);
            PayshopCommons::saveOrderReference($order, $instrument['reference']['fields']);

            $this->payshopUpdateOrder->execute(
                $orderId,
                "on-hold",
                $chargeId,
                $instrument['id']
            );

            return [
                'result' => 'success',
                'redirect' => $this->get_return_url($order)
            ];
        } catch (\Throwable $th) {
            wc_add_notice($th->getMessage(), 'error');
            return;
        }
    }
    
    /**
     * Create instrument
     * 
     * @param string $chargeId
     * @param WC_Order $order
     * @return array
     */
    private function createInstrument($chargeId)
    {
        $instrumentData = ["charge" => $chargeId];

        return $this->payshopCreateInstrument->execute($instrumentData);
    }
}

